var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsArrayShuffle } from 'tscommons-es-core';
import { commonsOutputDebug, commonsOutputDoing, commonsOutputInfo, commonsOutputResult, commonsOutputSuccess } from 'nodecommons-es-cli';
export function commonsAiCrossValidationBuildKs(trainingData, iterations) {
    const testingPivot = Math.min(0.2, 1 / iterations);
    const ks = [];
    const clone = [...trainingData];
    commonsArrayShuffle(clone);
    const testingLength = Math.floor(clone.length * testingPivot);
    for (let i = iterations; i-- > 0;) {
        const k = [...clone];
        const testingOffset = i * testingLength;
        const testing = k.splice(testingOffset, testingLength);
        const training = k;
        ks.push({
            training: training,
            testing: testing
        });
    }
    return ks;
}
export function commonsAiCrossValidationBuildEnumKs(keys, trainingData, iterations) {
    const datas = new Map();
    for (const key of keys) {
        datas.set(key, (trainingData.get(key) || [])
            .map((d) => ({
            input: d,
            output: key
        })));
    }
    const refactors = [];
    for (let i = iterations; i-- > 0;) {
        refactors.push({
            training: [],
            testing: []
        });
    }
    for (const key of keys) {
        const set = datas.get(key) || [];
        const k = commonsAiCrossValidationBuildKs(set, iterations);
        for (let i = iterations; i-- > 0;) {
            const training = k[i].training;
            const testing = k[i].testing;
            refactors[i].training.push(...training);
            refactors[i].testing.push(...testing);
        }
    }
    return refactors;
}
export function commonsAiCrossValidationTrain(crossValidationKs, modelConstructorCallback, epochs, earlyStopping) {
    return __awaiter(this, void 0, void 0, function* () {
        const iterations = crossValidationKs.length;
        const attempts = [];
        for (const crossValidationK of crossValidationKs) {
            if (crossValidationK.training.length === 0)
                throw new Error(`Not enough data to train for ${crossValidationKs.length} Ks.`);
            if (crossValidationK.testing.length === 0)
                throw new Error(`Not enough data to test for ${crossValidationKs.length} Ks.`);
            commonsOutputInfo(`K iteration ${attempts.length + 1} of ${iterations}`);
            const model = modelConstructorCallback(crossValidationK.training, epochs, crossValidationK.testing, earlyStopping);
            commonsOutputDoing('Building data');
            yield model.build();
            commonsOutputSuccess();
            commonsOutputDoing('Training model');
            const stats = yield model.train();
            commonsOutputResult(`loss=${stats.loss}, acc=${stats.acc}, epochs=${stats.epochs}`);
            commonsOutputDoing('Testing model quality');
            const quality = yield model.test();
            const delta = quality.positive - quality.negative;
            commonsOutputResult(`${quality.positive} positive, ${quality.negative} negative, overall ${delta}`);
            attempts.push({
                model: model,
                quality: quality,
                delta: delta
            });
        }
        attempts
            .sort((a, b) => {
            if (a.delta < b.delta)
                return -1;
            if (b.delta > a.delta)
                return 1;
            return 0;
        })
            .reverse();
        const qualities = attempts
            .map((attempt) => attempt.delta);
        commonsOutputDebug(`Attempt delta qualities are ${qualities.join(', ')}`);
        return {
            model: attempts[0].model,
            quality: attempts[0].quality
        };
    });
}
//# sourceMappingURL=commons-ai-cross-validator.js.map